@@ -14,13 +14,14 @@ import time
 from dataclasses import asdict
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Union
 
+import anyio
 from openai.types import CompletionUsage
 
 from vendor.tau2.data_model.message import AssistantMessage, UserMessage
 from vendor.tau2.user.user_simulator import UserSimulator
 
-from ...models import CompletionParams, EvaluationRow, InputMetadata, Message
-from ...types import MCPSession, MCPToolCall, TerminationReason, Trajectory
+from ...models import EvaluationRow, InputMetadata, Message, Status
+from ...types import TerminationReason, Trajectory, NonSkippableException
 
 if TYPE_CHECKING:
     from ..session.manager import GeneralMCPVectorEnv
@@ -34,7 +35,7 @@ class ExecutionManager:
     Manage rollout for MCP environments.
     """
 
-    async def execute_rollouts(
+    def execute_rollouts(
         self,
         envs: "GeneralMCPVectorEnv",
         policy: Union["LLMBasePolicy", Callable],
@@ -42,7 +43,7 @@ class ExecutionManager:
         openai_format_log_file: Optional[str] = None,
         max_concurrent_rollouts: int = 8,
         evaluation_rows: Optional[List[EvaluationRow]] = None,
-    ) -> List[EvaluationRow]:
+    ) -> List[asyncio.Task[EvaluationRow]]:
         """
         Execute general rollouts using tool calling interface with automatic record/playback.
 
@@ -65,7 +66,7 @@ class ExecutionManager:
             - Set and file exists: Playback mode (uses recorded data)
 
         Returns:
-            List of EvaluationRow objects with unified evaluation data format
+            List of asyncio.Task objects for external handling
         """
         start_time = time.time()
 
@@ -79,7 +80,7 @@ class ExecutionManager:
         elif playback_mode:
             logger.info(f"🎬 Playback mode: Using recorded data from {playback_file}")
         else:
-            logger.info(f"🚀 Live mode: No recording/playback")
+            logger.info("🚀 Live mode: No recording/playback")
 
         # Initialize OpenAI format logging for terminated trajectories only
         openai_logger = None
@@ -91,96 +92,64 @@ class ExecutionManager:
 
         logger.info(f"🧵 Starting {envs.n} rollouts with max {max_concurrent_rollouts} concurrent threads...")
 
-        results = {}
+        if evaluation_rows is None:
+            evaluation_rows = [EvaluationRow(messages=[], input_metadata=InputMetadata()) for _ in range(envs.n)]
+
+        shared_tool_schema = envs.tool_schemas
 
         semaphore = asyncio.Semaphore(max_concurrent_rollouts)
 
         async def _execute_with_semaphore(idx):
             async with semaphore:
-                result = await self._execute_rollout(
+                trajectory = await self._execute_rollout(
                     envs, policy, idx, steps, openai_logger, recording_mode, playback_mode, start_time
                 )
 
-                return result
+                # Convert trajectory to EvaluationRow immediately
+                evaluation_row: EvaluationRow = evaluation_rows[idx]
 
-        tasks = [_execute_with_semaphore(i) for i in range(envs.n)]
-        # exceptions will be try catched inside single _execute_rollout
-        trajectories = await asyncio.gather(*tasks)
+                # Handle multimodal content by extracting text from complex content structures
+                messages = []
+                for msg in trajectory.conversation_history:
+                    # Create a copy to avoid modifying the original
+                    msg_dict = dict(msg)
 
-        # Calculate durations
-        total_duration = time.time() - start_time
-        for trajectory in trajectories:
-            trajectory.duration = total_duration
+                    # Handle multimodal content (list of content blocks) by extracting text
+                    if isinstance(msg_dict.get("content"), list):
+                        text_content = None
+                        for content_block in msg_dict["content"]:
+                            if isinstance(content_block, dict) and content_block.get("type") == "text":
+                                text_content = content_block.get("text")
+                                break
+                        msg_dict["content"] = text_content or ""
 
-        shared_tool_schema = envs.tool_schemas
+                    messages.append(Message.model_validate(msg_dict))
 
-        # Enhanced reporting with control plane info
-        successful = sum(1 for traj in trajectories if traj.total_reward > 0)
-        terminated_by_control_plane = sum(
-            1
-            for traj in trajectories
-            if traj.control_plane_summary.get("termination_reason") == "control_plane_signal"
-        )
+                evaluation_row.messages = messages
+                evaluation_row.tools = shared_tool_schema
+                evaluation_row.usage = CompletionUsage(**trajectory.usage)
+                evaluation_row.input_metadata.completion_params = {
+                    "model": policy.model_id,
+                    "temperature": getattr(policy, "temperature", None),
+                    "max_tokens": getattr(policy, "max_tokens", None),
+                    "max_tool_calls": getattr(policy, "max_tools_per_turn", None),
+                }
 
-        logger.info(f"📊 Rollout complete: {successful}/{len(trajectories)} reached goal")
-        logger.info(f"🎛️  Control plane terminations: {terminated_by_control_plane}/{len(trajectories)}")
-        logger.info(f"⏱️  Total duration: {total_duration:.2f}s")
-        logger.info(f"🧵 Used {max_concurrent_rollouts} concurrent threads")
-
-        # Print log file locations if created
-        if openai_format_log_file:
-            logger.info(f"💬 OpenAI format log: {openai_format_log_file}")
-        if recording_mode:
-            logger.info(f"📝 Recorded trajectory: {playback_file}")
-            # Add note about control plane separation
-            logger.info(f"🎛️  Trajectories include control plane separation")
-
-        # Convert trajectories to unified EvaluationRow format. If no evaluation_rows are provided, create empty ones for backwards compatibility.
-        if evaluation_rows is None:
-            evaluation_rows = [EvaluationRow(messages=[], input_metadata=InputMetadata()) for _ in trajectories]
-
-        for idx, trajectory in enumerate(trajectories):
-            # Handle multimodal content by extracting text from complex content structures
-            messages = []
-            for msg in trajectory.conversation_history:
-                # Create a copy to avoid modifying the original
-                msg_dict = dict(msg)
-
-                # Handle multimodal content (list of content blocks) by extracting text
-                if isinstance(msg_dict.get("content"), list):
-                    text_content = None
-                    for content_block in msg_dict["content"]:
-                        if isinstance(content_block, dict) and content_block.get("type") == "text":
-                            text_content = content_block.get("text")
-                            break
-                    msg_dict["content"] = text_content or ""
-
-                messages.append(Message.model_validate(msg_dict))
-
-            evaluation_rows[idx].messages = messages
-            # evaluation_rows[idx].input_metadata.row_id = envs.dataset_rows[idx].id
-            # evaluation_rows[idx].input_metadata.dataset_info = asdict(envs.dataset_rows[idx])
-            evaluation_rows[idx].tools = shared_tool_schema
-            evaluation_rows[idx].usage = CompletionUsage(**trajectory.usage)
-            evaluation_rows[idx].input_metadata.completion_params = CompletionParams(
-                model=policy.model_id,
-                temperature=getattr(policy, "temperature", None),
-                max_tokens=getattr(policy, "max_tokens", None),
-                max_tool_calls=getattr(policy, "max_tools_per_turn", None),
-            )
-            if trajectory.terminated:
-                if trajectory.termination_reason == TerminationReason.ERROR:
-                    evaluation_rows[idx].rollout_status.status = "error"
-                    evaluation_rows[idx].rollout_status.termination_reason = trajectory.control_plane_summary.get(
-                        "error_message", None
+                if trajectory.terminated:
+                    extra_info = None
+                    if trajectory.control_plane_summary.get("error_message"):
+                        extra_info = {"error_message": trajectory.control_plane_summary.get("error_message")}
+                    evaluation_row.rollout_status = Status.rollout_finished(
+                        termination_reason=trajectory.termination_reason, extra_info=extra_info
                     )
                 else:
-                    evaluation_rows[idx].rollout_status.status = "finished"
-                    evaluation_rows[idx].rollout_status.termination_reason = trajectory.termination_reason
-            else:
-                evaluation_rows[idx].rollout_status.status = "running"
+                    evaluation_row.rollout_status = Status.rollout_running()
 
-        return evaluation_rows
+                return evaluation_row
+
+        # Create all tasks
+        tasks = [asyncio.create_task(_execute_with_semaphore(i)) for i in range(envs.n)]
+        return tasks
 
     async def _execute_rollout(
         self,
@@ -250,7 +219,7 @@ class ExecutionManager:
                 current_observation = user_message.content if user_message.content else ""
 
             user_prompt = envs.format_user_prompt(rollout_idx, current_observation)
-            conversation_history = [
+            trajectory.conversation_history = [
                 {"role": "system", "content": system_prompt},
                 {"role": "user", "content": user_prompt},
             ]
@@ -270,7 +239,7 @@ class ExecutionManager:
 
                 if user_simulator and user_simulator_state:
                     # Get user simulator messages and find the last assistant message
-                    user_simulator_messages = self._get_user_simulator_messages(conversation_history)
+                    user_simulator_messages = self._get_user_simulator_messages(trajectory.conversation_history)
 
                     # Last message was agent, simulated user response
                     if user_simulator_messages and isinstance(user_simulator_messages[-1], AssistantMessage):
@@ -281,7 +250,7 @@ class ExecutionManager:
                         user_content = user_message.content if user_message.content else ""
 
                         user_prompt = envs.format_user_prompt(rollout_idx, user_content)
-                        conversation_history.append({"role": "user", "content": user_prompt})
+                        trajectory.conversation_history.append({"role": "user", "content": user_prompt})
 
                         # Check if user simulator signaled termination
                         if UserSimulator.is_stop(user_message):
@@ -291,7 +260,7 @@ class ExecutionManager:
                 # In each turn: keep looping until assistant is ready to provide final response
                 while not turn_completed and not trajectory.terminated:
                     tool_calls, usage_stats, finish_reason = await policy(
-                        tool_schema, rollout_idx, conversation_history
+                        tool_schema, rollout_idx, trajectory.conversation_history
                     )
 
                     # calc llm usage stats happened in this turn if there is aany
@@ -314,7 +283,6 @@ class ExecutionManager:
 
                     # Execute each tool call sequentially
                     for tool_call in tool_calls:
-
                         # Execute tool call for this environment
                         observation, reward, env_end, info = await envs.step(rollout_idx, tool_call)
 
@@ -324,7 +292,7 @@ class ExecutionManager:
                             rollout_idx,
                             tool_call,
                             tool_response,
-                            conversation_history,
+                            trajectory.conversation_history,
                             reward,
                             env_end,
                             info,
@@ -355,12 +323,14 @@ class ExecutionManager:
                                 "num_tool_calls": 1,
                             }
                             print(f"🔍 control_plane_step: {control_plane_step}")
-                            conversation_history[-1]["control_plane_step"] = control_plane_step
+                            trajectory.conversation_history[-1]["control_plane_step"] = control_plane_step
                             trajectory.control_plane_steps.append(control_plane_step)
 
                             # Log conversation state for playback if in recording mode
                             if recording_mode:
-                                policy.log_conversation_state_for_playback(rollout_idx, step - 1, conversation_history)
+                                policy.log_conversation_state_for_playback(
+                                    rollout_idx, step - 1, trajectory.conversation_history
+                                )
 
                         if env_end:
                             # if the env marks the end of the rollout, break the tool call loop
@@ -394,17 +364,21 @@ class ExecutionManager:
                         "tool_calls": tool_calls_summary,
                         "num_tool_calls": len(tool_calls),
                     }
-                    conversation_history[-1]["control_plane_step"] = control_plane_step
+                    trajectory.conversation_history[-1]["control_plane_step"] = control_plane_step
                     trajectory.control_plane_steps.append(control_plane_step)
 
                     # Log conversation state for playback if in recording mode
                     if recording_mode:
-                        policy.log_conversation_state_for_playback(rollout_idx, step - 1, conversation_history)
+                        policy.log_conversation_state_for_playback(
+                            rollout_idx, step - 1, trajectory.conversation_history
+                        )
 
                 # if the env marks end, update control plane summary and do one last policy call, then break the agent loop
                 # this is to ensure each turn ends with an assistant message, which will align with the actual agentic llm behavior
                 if env_end:
-                    _, usage_stats, finish_reason = await policy(tool_schema, rollout_idx, conversation_history)
+                    _, usage_stats, finish_reason = await policy(
+                        tool_schema, rollout_idx, trajectory.conversation_history
+                    )
                     if usage_stats:
                         trajectory.usage["prompt_tokens"] += usage_stats.prompt_tokens
                         trajectory.usage["completion_tokens"] += usage_stats.completion_tokens
@@ -422,10 +396,10 @@ class ExecutionManager:
 
                     # Log final OpenAI conversation for terminated trajectories only
                     if openai_logger:
-                        if conversation_history and len(conversation_history) > 0:
+                        if trajectory.conversation_history and len(trajectory.conversation_history) > 0:
                             openai_logger(
                                 {
-                                    "messages": conversation_history,
+                                    "messages": trajectory.conversation_history,
                                     "metadata": {
                                         "session_id": session.session_id,
                                         "seed": session.seed,
@@ -451,8 +425,6 @@ class ExecutionManager:
             if not trajectory.termination_reason and step >= steps:
                 trajectory.termination_reason = TerminationReason.MAX_STEPS
 
-            trajectory.conversation_history = conversation_history
-
             # Add termination_reason to the final control_plane_step
             for msg in reversed(trajectory.conversation_history):
                 if msg.get("control_plane_step"):
@@ -462,26 +434,22 @@ class ExecutionManager:
             logger.info(
                 f"✅ Rollout {rollout_idx} completed: {trajectory.steps} steps, reward: {trajectory.total_reward:.2f}, termination: {trajectory.termination_reason}, in thread {threading.current_thread().name}"
             )
-
-        except asyncio.CancelledError:
-            logger.error(f"🚨 AsyncIO Cancel Error in roll out {rollout_idx}", exc_info=True)
-            failure_reason = "asyncio context cancelled"
-        except Exception as e:
-            logger.error(f"🚨 Error in rollout {rollout_idx}: {e}", exc_info=True)
-            failure_reason = str(e)
+        except NonSkippableException as e:
+            # terminate the rollout right away, no retry and preserve the current trajectory history.
+            # for other types of exceptions, keep propagate them to upper layers and handle them with retry handler.
+            trajectory.terminated = True
+            trajectory.termination_reason = TerminationReason.NON_SKIPPABLE_ERROR
+            trajectory.control_plane_summary.update({"error_message": str(e)})
+            logger.error(f"🚨 Rollout {rollout_idx} terminated due to non-skippable error: {str(e)}", exc_info=True)
         finally:
-            if failure_reason:
-                trajectory.terminated = True
-                trajectory.termination_reason = TerminationReason.ERROR
-                trajectory.control_plane_summary.update({"error_message": f"{failure_reason}"})
             try:
                 await envs.connection_manager.reset_session(session)
-            except:  # noqa: E722
-                logger.error(f"Error resetting session {session.session_id}")
+            except Exception as e:
+                logger.warning(f"Failed to reset session {session.session_id}: {type(e).__name__}: {e}", exc_info=True)
             try:
                 await envs.connection_manager.close_session(session)
-            except:  # noqa: E722
-                logger.error(f"Error closing session {session.session_id}")
+            except Exception as e:
+                logger.warning(f"Failed to close session {session.session_id}: {type(e).__name__}: {e}", exc_info=True)
         return trajectory
 
     async def _get_control_plane_status(self, session) -> Optional[Dict[str, Any]]:
