@@ -1,7 +1,8 @@
 import asyncio
 import json
+import logging
 import os
-from typing import Any, List, Optional, Union
+from typing import Any, AsyncIterator, List, Optional, Union
 
 from mcp.types import CallToolResult, TextContent
 from openai import NOT_GIVEN, NotGiven
@@ -12,8 +13,11 @@ from eval_protocol.dataset_logger.dataset_logger import DatasetLogger
 from eval_protocol.mcp.execution.policy import LiteLLMPolicy
 from eval_protocol.mcp.mcp_multi_client import MCPMultiClient
 from eval_protocol.models import EvaluationRow, Message
+from eval_protocol.pytest.rollout_processor import RolloutProcessor
 from eval_protocol.pytest.types import Dataset, RolloutProcessorConfig
 
+logger = logging.getLogger(__name__)
+
 
 class Agent:
     """
@@ -55,7 +59,7 @@ class Agent:
         self.append_message_and_log(message)
         if message.tool_calls:
             # Create tasks for all tool calls to run them in parallel
-            tool_tasks = []
+            tool_tasks: List[asyncio.Task[tuple[str, List[TextContent]]]] = []
             for tool_call in message.tool_calls:
                 tool_call_id = tool_call.id
                 tool_name = tool_call.function.name
@@ -67,18 +71,13 @@ class Agent:
                 tool_tasks.append(task)
 
             # Execute all tool calls in parallel
-            tool_results: List[tuple[str, List[TextContent]]] = await asyncio.gather(*tool_tasks)
+            tool_results = await asyncio.gather(*tool_tasks)
 
             # Add all tool results to messages (they will be in the same order as tool_calls)
             for tool_call, (tool_call_id, content) in zip(message.tool_calls, tool_results):
+                tool_message_content = self._format_tool_message_content(content)
                 self.append_message_and_log(
-                    Message(
-                        role="tool",
-                        content=[
-                            ChatCompletionContentPartTextParam(text=content.text, type="text") for content in content
-                        ],
-                        tool_call_id=tool_call_id,
-                    )
+                    Message(role="tool", content=tool_message_content, tool_call_id=tool_call_id)
                 )
             return await self.call_agent()
         return message.content
@@ -111,16 +110,49 @@ class Agent:
             raise NotImplementedError("Non-text content is not supported yet")
         return tool_result.content
 
+    def _format_tool_message_content(
+        self, content: List[TextContent]
+    ) -> Union[str, List[ChatCompletionContentPartTextParam]]:
+        """Format tool result content for inclusion in a tool message.
 
-async def default_agent_rollout_processor(
-    rows: List[EvaluationRow], config: RolloutProcessorConfig
-) -> List[EvaluationRow]:
-    dataset: Dataset = []
-    for row in rows:
-        agent = Agent(model=config.model, row=row, config_path=config.mcp_config_path, logger=config.logger)
-        await agent.setup()
-        await agent.call_agent()
-        dataset.append(agent.evaluation_row)
-        if agent.mcp_client:
-            await agent.mcp_client.cleanup()
-    return dataset
+        - If a single text item, return plain string per OpenAI semantics.
+        - If multiple items, return a list of text parts.
+        """
+        if len(content) == 1 and isinstance(content[0], TextContent):
+            return content[0].text
+        return [ChatCompletionContentPartTextParam(text=c.text, type="text") for c in content]
+
+
+class AgentRolloutProcessor(RolloutProcessor):
+    """Agent rollout processor for tool-calling agents."""
+
+    def __call__(self, rows: List[EvaluationRow], config: RolloutProcessorConfig) -> List[asyncio.Task[EvaluationRow]]:
+        """Create agent rollout tasks and return them for external handling."""
+
+        max_concurrent = getattr(config, "max_concurrent_rollouts", 8) or 8
+        semaphore = asyncio.Semaphore(max_concurrent)
+
+        async def process_row(row: EvaluationRow) -> EvaluationRow:
+            """Process a single row with agent rollout."""
+            agent = Agent(
+                model=row.input_metadata.completion_params["model"],
+                row=row,
+                config_path=config.mcp_config_path,
+                logger=config.logger,
+            )
+            try:
+                await agent.setup()
+                await agent.call_agent()
+                return agent.evaluation_row
+            finally:
+                if agent.mcp_client:
+                    await agent.mcp_client.cleanup()
+
+        async def _sem_wrapper(r: EvaluationRow) -> EvaluationRow:
+            async with semaphore:
+                result = await process_row(r)
+                return result
+
+        # Create and return tasks for external handling
+        tasks = [asyncio.create_task(_sem_wrapper(row)) for row in rows]
+        return tasks
