@@ -12,8 +12,8 @@ Usage:
     max_dataset_rows value set in the decorator).
 """
 
-import os
 import logging
+import os
 from typing import Optional
 
 
@@ -32,17 +32,13 @@ def pytest_addoption(parser) -> None:
         "--ep-print-summary",
         action="store_true",
         default=False,
-        help=(
-            "Print a concise summary line (suite/model/effort/agg score) at the end of each evaluation_test."
-        ),
+        help=("Print a concise summary line (suite/model/effort/agg score) at the end of each evaluation_test."),
     )
     group.addoption(
         "--ep-summary-json",
         action="store",
         default=None,
-        help=(
-            "Write a JSON summary artifact at the given path (e.g., ./outputs/aime_low.json)."
-        ),
+        help=("Write a JSON summary artifact at the given path (e.g., ./outputs/aime_low.json)."),
     )
     group.addoption(
         "--ep-input-param",
@@ -63,6 +59,23 @@ def pytest_addoption(parser) -> None:
             "Values: low|medium|high"
         ),
     )
+    group.addoption(
+        "--ep-max-retry",
+        action="store",
+        type=int,
+        default=0,
+        help=("Failed rollouts (with rollout_status.code indicating error) will be retried up to this many times."),
+    )
+    group.addoption(
+        "--ep-fail-on-max-retry",
+        action="store",
+        default="true",
+        choices=["true", "false"],
+        help=(
+            "Whether to fail the entire rollout when permanent failures occur after max retries. "
+            "Default: true (fail on permanent failures). Set to 'false' to continue with remaining rollouts."
+        ),
+    )
 
 
 def _normalize_max_rows(val: Optional[str]) -> Optional[str]:
@@ -104,10 +117,17 @@ def pytest_configure(config) -> None:
     if summary_json_path:
         os.environ["EP_SUMMARY_JSON"] = summary_json_path
 
+    max_retry = config.getoption("--ep-max-retry")
+    os.environ["EP_MAX_RETRY"] = str(max_retry)
+
+    fail_on_max_retry = config.getoption("--ep-fail-on-max-retry")
+    os.environ["EP_FAIL_ON_MAX_RETRY"] = fail_on_max_retry
+
     # Allow ad-hoc overrides of input params via CLI flags
     try:
         import json as _json
         import pathlib as _pathlib
+
         merged: dict = {}
         input_params_opts = config.getoption("--ep-input-param")
         if input_params_opts:
@@ -139,5 +159,3 @@ def pytest_configure(config) -> None:
     except Exception:
         # best effort, do not crash pytest session
         pass
-
-
