@@ -89,7 +89,7 @@ class MockPolicy:
             {
                 "role": "tool",
                 "content": response,
-                "tool_call_id": tool_call.tool_call_id or f"call_{len(conversation_history)-1}",
+                "tool_call_id": tool_call.tool_call_id or f"call_{len(conversation_history) - 1}",
                 "control_plane_step": {
                     "step": env_index,
                     "reward": reward,
@@ -149,7 +149,6 @@ class TestRolloutControlPlaneIntegration:
             patch.object(GeneralMCPVectorEnv, "close") as mock_close,
             patch.object(GeneralMCPVectorEnv, "format_user_prompt") as mock_format_user_prompt,
         ):
-
             # Setup mock vector environment
             mock_env = GeneralMCPVectorEnv(sessions, dataset_rows)
             mock_env.sessions = sessions
@@ -239,7 +238,11 @@ class TestRolloutControlPlaneIntegration:
             policy = MockPolicy(["right", "down", "right"])
 
             # Execute rollout
-            evaluation_rows = await self.execution_manager.execute_rollouts(mock_env, policy, steps=10)
+            tasks = self.execution_manager.execute_rollouts(mock_env, policy, steps=10)
+            evaluation_rows = []
+            for task in tasks:
+                row = await task
+                evaluation_rows.append(row)
 
             # Validate results
             assert len(evaluation_rows) == 1, "Should have one evaluation row"
@@ -265,9 +268,9 @@ class TestRolloutControlPlaneIntegration:
                     # Tool responses should only contain data plane information
                     content = msg.content or ""
                     # The content should not directly contain rewards or termination (they're in control_plane_step)
-                    assert (
-                        "reward" not in content.lower() or "reward_source" in content.lower()
-                    ), "Tool response should not directly contain reward"
+                    assert "reward" not in content.lower() or "reward_source" in content.lower(), (
+                        "Tool response should not directly contain reward"
+                    )
 
             # Validate control plane information from messages
             rewards = [msg.control_plane_step["reward"] for msg in messages_with_control_plane]
@@ -429,7 +432,6 @@ class TestRolloutControlPlaneIntegration:
             patch.object(GeneralMCPVectorEnv, "close") as mock_close,
             patch.object(GeneralMCPVectorEnv, "format_user_prompt") as mock_format_user_prompt,
         ):
-
             mock_env = GeneralMCPVectorEnv(sessions, dataset_rows)
             mock_env.sessions = sessions
             mock_env.dataset_rows = dataset_rows
@@ -457,7 +459,11 @@ class TestRolloutControlPlaneIntegration:
 
             # Execute rollout with control plane failure
             policy = MockPolicy(["right"])
-            evaluation_rows = await self.execution_manager.execute_rollouts(mock_env, policy, steps=1)
+            tasks = self.execution_manager.execute_rollouts(mock_env, policy, steps=1)
+            evaluation_rows = []
+            for task in tasks:
+                row = await task
+                evaluation_rows.append(row)
 
             # Should still work, but without control plane info
             assert len(evaluation_rows) == 1
@@ -493,22 +499,39 @@ class TestRolloutControlPlaneIntegration:
         policy = MockPolicy(["right"])
 
         with (
-            patch("eval_protocol.mcp_env.make", new_callable=AsyncMock) as mock_make,
+            patch("eval_protocol.mcp_env.make") as mock_make,
             patch("eval_protocol.mcp_env.ExecutionManager") as MockManager,
         ):
             mock_env = MagicMock()
             mock_make.return_value = mock_env
 
             manager_instance = MockManager.return_value
-            manager_instance.execute_rollouts = AsyncMock(return_value=["ok"])
 
-            result = await ep.rollout(
+            # Mock execute_rollouts to return tasks and track calls
+            call_args = []
+
+            async def mock_task():
+                return "ok"
+
+            def mock_execute_rollouts(*args, **kwargs):
+                call_args.append((args, kwargs))
+                import asyncio
+
+                return [asyncio.create_task(mock_task())]
+
+            manager_instance.execute_rollouts = mock_execute_rollouts
+
+            result = []
+            tasks = ep.rollout(
                 "http://localhost:1234/mcp/",
                 policy,
                 dataset=dataset,
                 model_id="test_model",
                 steps=5,
             )
+            for task in tasks:
+                row = await task
+                result.append(row)
 
             mock_make.assert_called_once_with(
                 "http://localhost:1234/mcp/",
@@ -517,14 +540,12 @@ class TestRolloutControlPlaneIntegration:
                 model_id="test_model",
             )
 
-            manager_instance.execute_rollouts.assert_called_once_with(
-                mock_make.return_value,
-                policy,
-                5,
-                None,
-                8,
-                None,
-            )
+            # Verify execute_rollouts was called with correct arguments
+            assert len(call_args) == 1, "execute_rollouts should be called once"
+            args, kwargs = call_args[0]
+            assert args[0] == mock_make.return_value, "First arg should be mock env"
+            assert args[1] == policy, "Second arg should be policy"
+            assert args[2] == 5, "Third arg should be steps"
 
             assert result == ["ok"]
 
